@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,11 +8,12 @@ import os
 import subprocess
 import hashlib
 import uuid
+from time import monotonic
 
 from xpra.child_reaper import getChildReaper
-from xpra.os_util import monotonic_time, bytestostr, strtobytes, umask_context, POSIX, WIN32
-from xpra.util import typedict, csv, nonl, envint, envbool, engs
-from xpra.scripts.config import parse_bool
+from xpra.os_util import bytestostr, umask_context, POSIX, WIN32
+from xpra.util import typedict, csv, envint, envbool, engs, net_utf8, u
+from xpra.scripts.config import parse_bool, parse_with_unit
 from xpra.simple_stats import std_unit
 from xpra.make_thread import start_thread
 from xpra.log import Logger
@@ -34,8 +35,16 @@ MIMETYPE_EXTS = {
                  }
 
 DENY = 0
-ACCEPT = 1
-OPEN = 2
+ACCEPT = 1      #the file / URL will be sent
+OPEN = 2        #don't send, open on sender
+
+def osclose(fd):
+    try:
+        os.close(fd)
+    except OSError as e:
+        filelog("os.close(%s)", fd, exc_info=True)
+        filelog.error("Error closing file download:")
+        filelog.error(" %s", e)
 
 def basename(filename):
     #we can't use os.path.basename,
@@ -69,10 +78,10 @@ def safe_open_download_file(basefilename, mimetype):
     root, ext = os.path.splitext(filename)
     base = 0
     while os.path.exists(filename):
-        filelog("cannot save file as %s: file already exists", filename)
+        filelog("cannot save file as %r: file already exists", filename)
         base += 1
         filename = root+("-%s" % base)+ext
-    filelog("safe_open_download_file(%s, %s) will use '%s'", basefilename, mimetype, filename)
+    filelog("safe_open_download_file(%s, %s) will use %r", basefilename, mimetype, filename)
     flags = os.O_CREAT | os.O_RDWR | os.O_EXCL
     try:
         flags |= os.O_BINARY                #@UndefinedVariable (win32 only)
@@ -105,8 +114,8 @@ class FileTransferAttributes:
         fta = pask(file_transfer)
         self.file_transfer_ask = fta and can_ask
         self.file_transfer = fta or pbool("file-transfer", file_transfer)
-        self.file_size_limit = file_size_limit
-        self.file_chunks = min(self.file_size_limit*1024*1024, FILE_CHUNKS_SIZE)
+        self.file_size_limit = parse_with_unit("file-size-limit", file_size_limit, "B", min_value=0)
+        self.file_chunks = FILE_CHUNKS_SIZE
         pa = pask(printing)
         self.printing_ask = pa and can_ask
         self.printing = pa or pbool("printing", printing)
@@ -119,6 +128,9 @@ class FileTransferAttributes:
         self.open_url = oua or pbool("open-url", open_url)
         self.file_ask_timeout = SEND_REQUEST_TIMEOUT
         self.open_command = open_command
+        self.files_requested = {}
+        self.files_accepted = {}
+        self.file_request_callback = {}
         filelog("file transfer attributes=%s", self.get_file_transfer_features())
 
     def get_file_transfer_features(self) -> dict:
@@ -126,7 +138,8 @@ class FileTransferAttributes:
         return {
                 "file-transfer"     : self.file_transfer,
                 "file-transfer-ask" : self.file_transfer_ask,
-                "file-size-limit"   : self.file_size_limit,
+                "file-size-limit"   : self.file_size_limit//1024//1024,     #legacy name (use max-file-size)
+                "max-file-size"     : self.file_size_limit,
                 "file-chunks"       : self.file_chunks,
                 "open-files"        : self.open_files,
                 "open-files-ask"    : self.open_files_ask,
@@ -211,22 +224,22 @@ class FileTransferHandler(FileTransferAttributes):
         self.remote_open_url = c.boolget("open-url")
         self.remote_open_url_ask = c.boolget("open-url-ask")
         self.remote_file_ask_timeout = c.intget("file-ask-timeout")
-        self.remote_file_size_limit = c.intget("file-size-limit")
-        self.remote_file_chunks = max(0, min(self.remote_file_size_limit*1024*1024, c.intget("file-chunks")))
+        self.remote_file_size_limit = c.intget("max-file-size") or c.intget("file-size-limit")*1024*1024
+        self.remote_file_chunks = max(0, min(self.remote_file_size_limit, c.intget("file-chunks")))
         self.dump_remote_caps()
 
     def dump_remote_caps(self):
-        filelog("file transfer remote caps: file-transfer=%s   (ask=%s)",
+        filelog("file transfer remote caps: file-transfer=%-5s   (ask=%s)",
                 self.remote_file_transfer, self.remote_file_transfer_ask)
-        filelog("file transfer remote caps: printing=%s        (ask=%s)",
+        filelog("file transfer remote caps: printing=%-5s        (ask=%s)",
                 self.remote_printing, self.remote_printing_ask)
-        filelog("file transfer remote caps: open-files=%s      (ask=%s)",
+        filelog("file transfer remote caps: open-files=%-5s      (ask=%s)",
                 self.remote_open_files, self.remote_open_files_ask)
-        filelog("file transfer remote caps: open-url=%s        (ask=%s)",
+        filelog("file transfer remote caps: open-url=%-5s        (ask=%s)",
                 self.remote_open_url, self.remote_open_url_ask)
 
     def get_info(self) -> dict:
-        info = FileTransferAttributes.get_info(self)
+        info = super().get_info()
         info["remote"] = {
             "file-transfer"     : self.remote_file_transfer,
             "file-transfer-ask" : self.remote_file_transfer_ask,
@@ -242,39 +255,83 @@ class FileTransferHandler(FileTransferAttributes):
             }
         return info
 
-    def check_digest(self, filename, digest, expected_digest, algo="sha1"):
-        if digest!=expected_digest:
-            filelog.error("Error: data does not match, invalid %s file digest for '%s'", algo, filename)
-            filelog.error(" received %s, expected %s", digest, expected_digest)
-            raise Exception("failed %s digest verification" % algo)
-        else:
-            filelog("%s digest matches: %s", algo, digest)
+
+    def digest_mismatch(self, filename, digest, expected_digest, algo="sha1"):
+        filelog.error("Error: data does not match, invalid %s file digest for '%s'", algo, filename)
+        filelog.error(" received %s, expected %s", digest, expected_digest)
+        raise Exception("failed %s digest verification" % algo)
 
 
     def _check_chunk_receiving(self, chunk_id, chunk_no):
         chunk_state = self.receive_chunks_in_progress.get(chunk_id)
         filelog("_check_chunk_receiving(%s, %s) chunk_state=%s", chunk_id, chunk_no, chunk_state)
         if chunk_state:
+            if chunk_state[-4]:
+                #transfer has been cancelled
+                return
             chunk_state[-2] = 0     #this timer has been used
             if chunk_state[-1]==0:
-                filelog.error("Error: chunked file transfer timed out")
-                del self.receive_chunks_in_progress[chunk_id]
+                filelog.error("Error: chunked file transfer '%s' timed out", chunk_id)
+                self.receive_chunks_in_progress.pop(chunk_id, None)
+
+    def cancel_download(self, send_id, message="Cancelled"):
+        filelog("cancel_download(%s, %s)", send_id, message)
+        for chunk_id, chunk_state in dict(self.receive_chunks_in_progress).items():
+            if chunk_state[-3]==send_id:
+                self.cancel_file(chunk_id, message)
+                return
+        filelog.error("Error: cannot cancel download %s, entry not found!", u(send_id))
+
+    def cancel_file(self, chunk_id, message, chunk=0):
+        filelog("cancel_file%s", (chunk_id, message, chunk))
+        chunk_state = self.receive_chunks_in_progress.get(chunk_id)
+        if chunk_state:
+            #mark it as cancelled:
+            chunk_state[-4] = True
+            timer = chunk_state[-2]
+            if timer:
+                chunk_state[-2] = 0
+                self.source_remove(timer)
+            fd = chunk_state[1]
+            osclose(fd)
+            #remove this transfer after a little while,
+            #so in-flight packets won't cause errors
+            def clean_receive_state():
+                self.receive_chunks_in_progress.pop(chunk_id, None)
+                return False
+            self.timeout_add(20000, clean_receive_state)
+            filename = chunk_state[2]
+            try:
+                os.unlink(filename)
+            except OSError as e:
+                filelog("os.unlink(%s)", filename, exc_info=True)
+                filelog.error("Error: failed to delete temporary download file")
+                filelog.error(" '%s' : %s", filename, e)
+        self.send("ack-file-chunk", chunk_id, False, message, chunk)
 
     def _process_send_file_chunk(self, packet):
         chunk_id, chunk, file_data, has_more = packet[1:5]
-        chunk_id = bytestostr(chunk_id)
+        chunk_id = net_utf8(chunk_id)
         filelog("_process_send_file_chunk%s", (chunk_id, chunk, "%i bytes" % len(file_data), has_more))
         chunk_state = self.receive_chunks_in_progress.get(chunk_id)
         if not chunk_state:
-            filelog.error("Error: cannot find the file transfer id '%s'", nonl(chunk_id))
-            self.send("ack-file-chunk", chunk_id, False, "file transfer id not found", chunk)
+            filelog.error("Error: cannot find the file transfer id '%r'", chunk_id)
+            self.cancel_file(chunk_id, "file transfer id %r not found" % chunk_id, chunk)
             return
+        if chunk_state[-4]:
+            filelog("got chunk for a cancelled file transfer, ignoring it")
+            return
+        def progress(position, error=None):
+            start = chunk_state[0]
+            send_id = chunk_state[-3]
+            filesize = chunk_state[6]
+            self.transfer_progress_update(False, send_id, monotonic()-start, position, filesize, error)
         fd = chunk_state[1]
         if chunk_state[-1]+1!=chunk:
             filelog.error("Error: chunk number mismatch, expected %i but got %i", chunk_state[-1]+1, chunk)
-            self.send("ack-file-chunk", chunk_id, False, "chunk number mismatch", chunk)
-            del self.receive_chunks_in_progress[chunk_id]
-            os.close(fd)
+            self.cancel_file(chunk_id, "chunk number mismatch", chunk)
+            osclose(fd)
+            progress(-1, "chunk no mismatch")
             return
         #update chunk number:
         chunk_state[-1] = chunk
@@ -288,15 +345,13 @@ class FileTransferHandler(FileTransferAttributes):
         except OSError as e:
             filelog.error("Error: cannot write file chunk")
             filelog.error(" %s", e)
-            self.send("ack-file-chunk", chunk_id, False, "write error: %s" % e, chunk)
-            del self.receive_chunks_in_progress[chunk_id]
-            try:
-                os.close(fd)
-            except OSError:
-                pass
+            self.cancel_file(chunk_id, "write error: %s" % e, chunk)
+            osclose(fd)
+            progress(-1, "write error (%s)" % e)
             return
         self.send("ack-file-chunk", chunk_id, True, "", chunk)
         if has_more:
+            progress(written)
             timer = chunk_state[-2]
             if timer:
                 self.source_remove(timer)
@@ -304,18 +359,23 @@ class FileTransferHandler(FileTransferAttributes):
             timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_receiving, chunk_id, chunk)
             chunk_state[-2] = timer
             return
-        del self.receive_chunks_in_progress[chunk_id]
-        os.close(fd)
+        self.receive_chunks_in_progress.pop(chunk_id, None)
+        osclose(fd)
         #check file size and digest then process it:
         filename, mimetype, printit, openit, filesize, options = chunk_state[2:8]
         if written!=filesize:
             filelog.error("Error: expected a file of %i bytes, got %i", filesize, written)
+            progress(-1, "file size mismatch")
             return
-        expected_digest = options.get("sha1")
-        if expected_digest:
-            self.check_digest(filename, digest.hexdigest(), expected_digest)
+        expected_digest = options.strget("sha1")
+        if expected_digest and digest.hexdigest()!=expected_digest:
+            progress(-1, "checksum mismatch")
+            self.digest_mismatch(filename, digest, expected_digest, "sha1")
+            return
+
+        progress(written)
         start_time = chunk_state[0]
-        elapsed = monotonic_time()-start_time
+        elapsed = monotonic()-start_time
         mimetype = bytestostr(mimetype)
         filelog("%i bytes received in %i chunks, took %ims", filesize, chunk, elapsed*1000)
         self.process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
@@ -330,10 +390,15 @@ class FileTransferHandler(FileTransferAttributes):
                 self.file_transfer, self.file_transfer_ask)
         filelog("accept_data: open-files=%s, open-files-ask=%s",
                 self.open_files, self.open_files_ask)
-        if not self.file_transfer or self.file_transfer_ask:
+        req = self.files_accepted.pop(send_id, None)
+        filelog("accept_data: files_accepted[%s]=%s", send_id, req)
+        if req is not None:
+            return (False, req)
+        if printit:
+            if not self.printing or self.printing_ask:
+                printit = False
+        elif not self.file_transfer or self.file_transfer_ask:
             return None
-        if printit and (not self.printing or self.printing_ask):
-            printit = False
         if openit and (not self.open_files or self.open_files_ask):
             #we can't ask in this implementation,
             #so deny the request to open it:
@@ -342,14 +407,25 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_file(self, packet):
         #the remote end is sending us a file
-        start = monotonic_time()
+        start = monotonic()
         basefilename, mimetype, printit, openit, filesize, file_data, options = packet[1:8]
         send_id = ""
         if len(packet)>=9:
-            send_id = packet[8]
-        r = self.accept_data(send_id, b"file", basefilename, printit, openit)
+            send_id = net_utf8(packet[8])
+        #basefilename should be utf8:
+        basefilename = net_utf8(basefilename)
+        mimetype = net_utf8(mimetype)
+        if filesize<=0:
+            filelog.error("Error: invalid file size: %s", filesize)
+            filelog.error(" file transfer aborted for %r", basefilename)
+            return
+        args = (send_id, "file", basefilename, printit, openit)
+        r = self.accept_data(*args)
+        filelog("%s%s=%s", self.accept_data, args, r)
         if r is None:
-            filelog.warn("Warning: file transfer rejected for file '%s'", bytestostr(basefilename))
+            filelog.warn("Warning: %s rejected for file '%s'",
+                         ("transfer", "printing")[bool(printit)],
+                         basefilename)
             return
         #accept_data can override the flags:
         printit, openit = r
@@ -362,21 +438,16 @@ class FileTransferHandler(FileTransferAttributes):
             assert self.file_transfer
         l("receiving file: %s",
           [basefilename, mimetype, printit, openit, filesize, "%s bytes" % len(file_data), options])
-        assert filesize>0, "invalid file size: %s" % filesize
-        if filesize>self.file_size_limit*1024*1024:
+        if filesize>self.file_size_limit:
             l.error("Error: file '%s' is too large:", basefilename)
-            l.error(" %iMB, the file size limit is %iMB", filesize//1024//1024, self.file_size_limit)
+            l.error(" %sB, the file size limit is %sB",
+                    std_unit(filesize), std_unit(self.file_size_limit))
             return
         chunk_id = options.strget("file-chunk-id")
-        #basefilename should be utf8:
         try:
-            base = basefilename.decode("utf8")
-        except UnicodeDecodeError:
-            base = bytestostr(basefilename)
-        try:
-            filename, fd = safe_open_download_file(base, mimetype)
+            filename, fd = safe_open_download_file(basefilename, mimetype)
         except OSError as e:
-            filelog("cannot save file %s / %s", base, mimetype, exc_info=True)
+            filelog("cannot save file %s / %s", basefilename, mimetype, exc_info=True)
             filelog.error("Error: failed to save downloaded file")
             filelog.error(" %s", e)
             if chunk_id:
@@ -389,14 +460,15 @@ class FileTransferHandler(FileTransferAttributes):
                 self.send("ack-file-chunk", chunk_id, False, "too many file transfers in progress: %i" % l, 0)
                 os.close(fd)
                 return
-            digest = hashlib.sha1()
+            digest = hashlib.sha256()
             chunk = 0
             timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_receiving, chunk_id, chunk)
             chunk_state = [
-                monotonic_time(),
+                monotonic(),
                 fd, filename, mimetype,
                 printit, openit, filesize,
-                options, digest, 0, timer, chunk,
+                options, digest, 0, False, send_id,
+                timer, chunk,
                 ]
             self.receive_chunks_in_progress[chunk_id] = chunk_state
             self.send("ack-file-chunk", chunk_id, True, "", chunk)
@@ -408,33 +480,45 @@ class FileTransferHandler(FileTransferAttributes):
             l.error(" received %i bytes, expected %i bytes", len(file_data), filesize)
             return
         #check digest if present:
-        def check_digest(algo="sha1", libfn=hashlib.sha1):
+        def check_digest(algo="sha256", libfn=hashlib.sha256):
             digest = options.get(algo)
             if digest:
-                u = libfn()
-                u.update(file_data)
-                l("%s digest: %s - expected: %s", algo, u.hexdigest(), digest)
-                self.check_digest(basefilename, u.hexdigest(), digest, algo)
+                h = libfn()
+                h.update(file_data)
+                l("%s digest: %s - expected: %s", algo, h.hexdigest(), digest)
+                if digest!=h.hexdigest():
+                    self.digest_mismatch(filename, digest, h.hexdigest(), algo)
+        check_digest("sha256", hashlib.sha256)
         check_digest("sha1", hashlib.sha1)
         check_digest("md5", hashlib.md5)
         try:
             os.write(fd, file_data)
         finally:
             os.close(fd)
-        self.transfer_progress_update(False, send_id, monotonic_time()-start, filesize, filesize, None)
+        self.transfer_progress_update(False, send_id, monotonic()-start, filesize, filesize, None)
         self.process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
 
 
     def process_downloaded_file(self, filename, mimetype, printit, openit, filesize, options):
-        t = start_thread(self.do_process_downloaded_file, "process-download", daemon=False,
-                         args=(filename, mimetype, printit, openit, filesize, options))
-        filelog("started process-download thread: %s", t)
-
-    def do_process_downloaded_file(self, filename, mimetype, printit, openit, filesize, options):
-        filelog("do_process_downloaded_file%s", (filename, mimetype, printit, openit, filesize, options))
         filelog.info("downloaded %s bytes to %s file%s:",
                      filesize, (mimetype or "temporary"), ["", " for printing"][int(printit)])
         filelog.info(" '%s'", filename)
+        #some file requests may have a custom callback
+        #(ie: bug report tool will just include the file)
+        rf = options.tupleget("request-file")
+        if rf and len(rf)>=2:
+            argf = rf[0]
+            cb = self.file_request_callback.pop(bytestostr(argf), None)
+            if cb:
+                cb(filename, filesize)
+                return
+        if printit or openit:
+            t = start_thread(self.do_process_downloaded_file, "process-download", daemon=False,
+                             args=(filename, mimetype, printit, openit, filesize, options))
+            filelog("started process-download thread: %s", t)
+
+    def do_process_downloaded_file(self, filename, mimetype, printit, openit, filesize, options):
+        filelog("do_process_downloaded_file%s", (filename, mimetype, printit, openit, filesize, options))
         if printit:
             self._print_file(filename, mimetype, options)
             return
@@ -488,14 +572,14 @@ class FileTransferHandler(FileTransferAttributes):
             printlog("printing failed and returned %i", job)
             delfile()
             return
-        start = monotonic_time()
+        start = monotonic()
         def check_printing_finished():
             done = printing_finished(job)
             printlog("printing_finished(%s)=%s", job, done)
             if done:
                 delfile()
                 return False
-            if monotonic_time()-start>=PRINT_JOB_TIMEOUT:
+            if monotonic()-start>=PRINT_JOB_TIMEOUT:
                 printlog.warn("Warning: print job %s timed out", job)
                 delfile()
                 return False
@@ -553,15 +637,15 @@ class FileTransferHandler(FileTransferAttributes):
 
     def file_size_warning(self, action, location, basefilename, filesize, limit):
         filelog.warn("Warning: cannot %s the file '%s'", action, basefilename)
-        filelog.warn(" this file is too large: %sB", std_unit(filesize, unit=1024))
-        filelog.warn(" the %s file size limit is %iMB", location, limit)
+        filelog.warn(" this file is too large: %sB", std_unit(filesize))
+        filelog.warn(" the %s file size limit is %sB", location, std_unit(limit))
 
     def check_file_size(self, action, filename, filesize):
         basefilename = os.path.basename(filename)
-        if filesize>self.file_size_limit*1024*1024:
+        if filesize>self.file_size_limit:
             self.file_size_warning(action, "local", basefilename, filesize, self.file_size_limit)
             return False
-        if filesize>self.remote_file_size_limit*1024*1024:
+        if filesize>self.remote_file_size_limit:
             self.file_size_warning(action, "remote", basefilename, filesize, self.remote_file_size_limit)
             return False
         return True
@@ -569,15 +653,17 @@ class FileTransferHandler(FileTransferAttributes):
 
     def send_request_file(self, filename, openit=True):
         self.send("request-file", filename, openit)
+        self.files_requested[filename] = openit
 
 
     def _process_open_url(self, packet):
-        url, send_id = packet[1:3]
+        send_id = net_utf8(packet[2])
+        url = net_utf8(packet[1])
         if not self.open_url:
             filelog.warn("Warning: received a request to open URL '%s'", url)
             filelog.warn(" but opening of URLs is disabled")
             return
-        if not self.open_url_ask or self.accept_data(send_id, b"url", url, False, True):
+        if not self.open_url_ask or self.accept_data(send_id, "url", url, False, True):
             self._open_url(url)
         else:
             filelog("url '%s' not accepted", url)
@@ -589,14 +675,14 @@ class FileTransferHandler(FileTransferAttributes):
             return False
         if self.remote_open_url_ask:
             #ask the client if it is OK to send
-            return self.send_data_request("open", b"url", url)
+            return self.send_data_request("open", "url", url)
         self.do_send_open_url(url)
         return True
 
     def do_send_open_url(self, url, send_id=""):
         self.send("open-url", url, send_id)
 
-    def send_file(self, filename, mimetype, data, filesize=0, printit=False, openit=False, options={}):
+    def send_file(self, filename, mimetype, data, filesize=0, printit=False, openit=False, options=None):
         if printit:
             l = printlog
             if not self.printing:
@@ -634,33 +720,55 @@ class FileTransferHandler(FileTransferAttributes):
         if not self.check_file_size(action, filename, filesize):
             return False
         if ask:
-            return self.send_data_request(action, b"file", filename, mimetype, data, filesize, printit, openit, options)
+            return self.send_data_request(action, "file", filename, mimetype, data, filesize, printit, openit, options)
         self.do_send_file(filename, mimetype, data, filesize, printit, openit, options)
         return True
 
-    def send_data_request(self, action, dtype, url, mimetype="", data="", filesize=0, printit=False, openit=True, options={}):
+    def send_data_request(self, action, dtype, url, mimetype="", data="", filesize=0, printit=False, openit=True, options=None):
         send_id = uuid.uuid4().hex
         if len(self.pending_send_data)>=MAX_CONCURRENT_FILES:
             filelog.warn("Warning: %s dropped", action)
             filelog.warn(" %i transfer%s already waiting for a response",
                          len(self.pending_send_data), engs(self.pending_send_data))
             return None
-        self.pending_send_data[send_id] = (dtype, url, mimetype, data, filesize, printit, openit, options)
+        self.pending_send_data[send_id] = (dtype, url, mimetype, data, filesize, printit, openit, options or {})
         delay = self.remote_file_ask_timeout*1000
         self.pending_send_data_timers[send_id] = self.timeout_add(delay, self.send_data_ask_timeout, send_id)
         filelog("sending data request for %s '%s' with send-id=%s",
-                bytestostr(dtype), url, send_id)
-        self.send("send-data-request", dtype, send_id, url, mimetype, filesize, printit, openit)
+                u(dtype), u(url), send_id)
+        self.send("send-data-request", dtype, send_id, url, mimetype, filesize, printit, openit, options or {})
         return send_id
 
 
     def _process_send_data_request(self, packet):
         dtype, send_id, url, _, filesize, printit, openit = packet[1:8]
-        filelog("process send-data-request: send_id=%s, url=%s, printit=%s, openit=%s", send_id, url, printit, openit)
+        options = {}
+        if len(packet)>=9:
+            options = packet[8]
+        #filenames and url are always sent encoded as utf8:
+        url = net_utf8(url)
+        dtype = net_utf8(dtype)
+        send_id = net_utf8(send_id)
+        self.do_process_send_data_request(dtype, send_id, url, _, filesize, printit, openit, typedict(options))
+
+
+    def do_process_send_data_request(self, dtype, send_id, url, _, filesize, printit, openit, options):
+        filelog("do_process_send_data_request: send_id=%s, url=%s, printit=%s, openit=%s, options=%s",
+                u(send_id), url, printit, openit, options)
         def cb_answer(accept):
             filelog("accept%s=%s", (url, printit, openit), accept)
             self.send("send-data-response", send_id, accept)
-        if dtype==b"file":
+        #could be a request we made:
+        #(in which case we can just accept it without prompt)
+        rf = options.tupleget("request-file")
+        if rf and len(rf)>=2:
+            argf, openit = rf[:2]
+            openit = self.files_requested.pop(bytestostr(argf), None)
+            if openit is not None:
+                self.files_accepted[send_id] = openit
+                cb_answer(True)
+                return
+        if dtype=="file":
             if not self.file_transfer:
                 cb_answer(False)
                 return
@@ -671,7 +779,7 @@ class FileTransferHandler(FileTransferAttributes):
                 ask = self.file_transfer_ask or self.open_files_ask
             else:
                 ask = self.file_transfer_ask
-        elif dtype==b"url":
+        elif dtype=="url":
             if not self.open_url:
                 cb_answer(False)
                 return
@@ -697,29 +805,22 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_data_response(self, packet):
         send_id, accept = packet[1:3]
+        send_id = net_utf8(send_id)
         filelog("process send-data-response: send_id=%s, accept=%s", send_id, accept)
-        timer = self.pending_send_data_timers.get(send_id)
+        timer = self.pending_send_data_timers.pop(send_id, None)
         if timer:
-            try:
-                del self.pending_send_data_timers[send_id]
-            except KeyError:
-                pass
             self.source_remove(timer)
-        v = self.pending_send_data.get(bytestostr(send_id))
-        if not v:
+        v = self.pending_send_data.pop(send_id, None)
+        if v is None:
             filelog.warn("Warning: cannot find send-file entry")
             return
-        try:
-            del self.pending_send_data[send_id]
-        except KeyError:
-            pass
-        dtype = v[0]
-        url = v[1]
+        dtype = net_utf8(v[0])
+        url = net_utf8(v[1])
         if accept==DENY:
-            filelog.info("the request to send %s '%s' has been denied", bytestostr(dtype), url)
+            filelog.info("the request to send %s '%s' has been denied", dtype, url)
             return
         assert accept in (ACCEPT, OPEN), "unknown value for send-data response: %s" % (accept,)
-        if dtype==b"file":
+        if dtype=="file":
             mimetype, data, filesize, printit, openit, options = v[2:]
             if accept==ACCEPT:
                 self.do_send_file(url, mimetype, data, filesize, printit, openit, options, send_id)
@@ -727,7 +828,7 @@ class FileTransferHandler(FileTransferAttributes):
                 assert openit and accept==OPEN
                 #try to open at this end:
                 self._open_file(url)
-        elif dtype==b"url":
+        elif dtype=="url":
             if accept==ACCEPT:
                 self.do_send_open_url(url, send_id)
             else:
@@ -739,10 +840,7 @@ class FileTransferHandler(FileTransferAttributes):
 
     def send_data_ask_timeout(self, send_id):
         v = self.pending_send_data.pop(send_id, None)
-        try:
-            del self.pending_send_data_timers[send_id]
-        except KeyError:
-            pass
+        self.pending_send_data_timers.pop(send_id, None)
         if not v:
             filelog.warn("Warning: send timeout, id '%s' not found!", send_id)
             return False
@@ -752,21 +850,22 @@ class FileTransferHandler(FileTransferAttributes):
         filelog.warn(" the send approval request timed out")
         return False
 
-    def do_send_file(self, filename, mimetype, data, filesize=0, printit=False, openit=False, options={}, send_id=""):
+    def do_send_file(self, filename, mimetype, data, filesize=0, printit=False, openit=False, options=None, send_id=""):
         if printit:
             action = "print"
             l = printlog
         else:
             action = "upload"
             l = filelog
-        l("do_send_file%s", (filename, mimetype, type(data), "%i bytes" % filesize, printit, openit, options))
+        l("do_send_file%s", (u(filename), mimetype, type(data), "%i bytes" % filesize, printit, openit, options))
         if not self.check_file_size(action, filename, filesize):
             return False
-        u = hashlib.sha1()
-        u.update(data)
+        h = hashlib.sha256()
+        h.update(data)
         absfile = os.path.abspath(filename)
-        filelog("sha1 digest(%s)=%s", absfile, u.hexdigest())
-        options["sha1"] = u.hexdigest()
+        filelog("sha256 digest('%s')=%s", u(absfile), h.hexdigest())
+        options = options or {}
+        options["sha256"] = h.hexdigest()
         chunk_size = min(self.file_chunks, self.remote_file_chunks)
         if 0<chunk_size<filesize:
             if len(self.send_chunks_in_progress)>=MAX_CONCURRENT_FILES:
@@ -776,60 +875,65 @@ class FileTransferHandler(FileTransferAttributes):
             options["file-chunk-id"] = chunk_id
             #timer to check that the other end is requesting more chunks:
             timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_sending, chunk_id, 0)
-            chunk_state = [monotonic_time(), data, chunk_size, timer, 0]
+            chunk_state = [monotonic(), data, chunk_size, timer, 0]
             self.send_chunks_in_progress[chunk_id] = chunk_state
             cdata = ""
+            filelog("using chunks, sending initial file-chunk-id=%s, for chunk size=%s",
+                    chunk_id, chunk_size)
         else:
             #send everything now:
             cdata = self.compressed_wrapper("file-data", data)
             assert len(cdata)<=filesize     #compressed wrapper ensures this is true
+            filelog("sending full file: %i bytes (chunk size=%i)", filesize, chunk_size)
         basefilename = os.path.basename(filename)
-        #convert str to utf8 bytes:
-        try:
-            base = basefilename.encode("utf8")
-        except (AttributeError, UnicodeEncodeError):
-            base = strtobytes(basefilename)
-        self.send("send-file", base, mimetype, printit, openit, filesize, cdata, options, send_id)
+        self.send("send-file", basefilename, mimetype, printit, openit, filesize, cdata, options, send_id)
         return True
 
     def _check_chunk_sending(self, chunk_id, chunk_no):
         chunk_state = self.send_chunks_in_progress.get(chunk_id)
         filelog("_check_chunk_sending(%s, %s) chunk_state found: %s", chunk_id, chunk_no, bool(chunk_state))
         if chunk_state:
-            chunk_state[-2] = 0         #timer has fired
+            chunk_state[3] = 0         #timer has fired
             if chunk_state[-1]==chunk_no:
-                filelog.error("Error: chunked file transfer timed out on chunk %i", chunk_no)
-                try:
-                    del self.send_chunks_in_progress[chunk_id]
-                except KeyError:
-                    pass
+                filelog.error("Error: chunked file transfer '%s' timed out", chunk_id)
+                filelog.error(" on chunk %i", chunk_no)
+                self.cancel_sending(chunk_id)
+
+    def cancel_sending(self, chunk_id):
+        chunk_state = self.send_chunks_in_progress.pop(chunk_id, None)
+        filelog("cancel_sending(%s) chunk state found: %s", chunk_id, bool(chunk_state))
+        if chunk_state:
+            timer = chunk_state[3]
+            if timer:
+                chunk_state[3] = 0
+                self.source_remove(timer)
 
     def _process_ack_file_chunk(self, packet):
         #the other end received our send-file or send-file-chunk,
         #send some more file data
         filelog("ack-file-chunk: %s", packet[1:])
         chunk_id, state, error_message, chunk = packet[1:5]
-        chunk_id = bytestostr(chunk_id)
+        chunk_id = net_utf8(chunk_id)
         if not state:
-            filelog.error("Error: remote end is cancelling the file transfer:")
-            filelog.error(" %s", error_message)
-            del self.send_chunks_in_progress[chunk_id]
+            filelog.info("the remote end is cancelling the file transfer:")
+            filelog.info(" %s", net_utf8(error_message))
+            self.cancel_sending(chunk_id)
             return
         chunk_state = self.send_chunks_in_progress.get(chunk_id)
         if not chunk_state:
-            filelog.error("Error: cannot find the file transfer id '%s'", nonl(chunk_id))
+            filelog.error("Error: cannot find the file transfer id '%r'", chunk_id)
             return
         if chunk_state[-1]!=chunk:
             filelog.error("Error: chunk number mismatch (%i vs %i)", chunk_state, chunk)
-            del self.send_chunks_in_progress[chunk_id]
+            self.cancel_sending(chunk_id)
             return
         start_time, data, chunk_size, timer, chunk = chunk_state
         if not data:
             #all sent!
-            elapsed = monotonic_time()-start_time
+            elapsed = monotonic()-start_time
             filelog("%i chunks of %i bytes sent in %ims (%sB/s)",
                     chunk, chunk_size, elapsed*1000, std_unit(chunk*chunk_size/elapsed))
-            del self.send_chunks_in_progress[chunk_id]
+            self.cancel_sending(chunk_id)
             return
         assert chunk_size>0
         #carve out another chunk:
@@ -847,3 +951,7 @@ class FileTransferHandler(FileTransferAttributes):
 
     def compressed_wrapper(self, datatype, data, level=5):
         raise NotImplementedError()
+
+    def transfer_progress_update(self, send=True, transfer_id=0, elapsed=0, position=0, total=0, error=None):
+        #this method is overriden in the gtk client:
+        filelog("transfer_progress_update%s", (send, transfer_id, elapsed, position, total, error))
