@@ -9,11 +9,12 @@
 # but it works on win32, for whatever that's worth.
 
 import os
+from time import monotonic
 from socket import error as socket_error
-from threading import Lock, Event
+from threading import Lock, RLock, Event
 from queue import Queue
 
-from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, hexstr, monotonic_time
+from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, hexstr
 from xpra.util import repr_ellipsized, ellipsizer, csv, envint, envbool, typedict
 from xpra.make_thread import make_thread, start_thread
 from xpra.net.common import (
@@ -67,10 +68,10 @@ def force_flush_queue(q):
             while q.qsize()>0:
                 q.read(False)
         except Exception:
-            pass
+            log("force_flush_queue(%s)", q, exc_info=True)
         q.put_nowait(None)
     except Exception:
-        pass
+        log("force_flush_queue(%s)", q, exc_info=True)
 
 
 def verify_packet(packet):
@@ -109,6 +110,10 @@ def do_verify_packet(tree, packet):
         r = False
     return r
 
+CONNECTION_LOST = "connection-lost"
+GIBBERISH = "gibberish"
+INVALID = "invalid"
+
 
 class Protocol:
     """
@@ -117,10 +122,6 @@ class Protocol:
         and decompress and decode when receiving.
     """
 
-    CONNECTION_LOST = "connection-lost"
-    GIBBERISH = "gibberish"
-    INVALID = "invalid"
-
     TYPE = "xpra"
 
     def __init__(self, scheduler, conn, process_packet_cb, get_packet_cb=None):
@@ -129,7 +130,7 @@ class Protocol:
         """
         assert scheduler is not None
         assert conn is not None
-        self.start_time = monotonic_time()
+        self.start_time = monotonic()
         self.timeout_add = scheduler.timeout_add
         self.idle_add = scheduler.idle_add
         self.source_remove = scheduler.source_remove
@@ -161,7 +162,7 @@ class Protocol:
         #initial value which may get increased by client/server after handshake:
         self.max_packet_size = MAX_PACKET_SIZE
         self.abs_max_packet_size = 256*1024*1024
-        self.large_packets = [b"hello", b"window-metadata", b"sound-data", b"notify_show", b"setting-change", b"shell-reply"]
+        self.large_packets = ["hello", "window-metadata", "sound-data", "notify_show", "setting-change", "shell-reply"]
         self.send_aliases = {}
         self.send_flush_flag = False
         self.receive_aliases = {}
@@ -180,6 +181,7 @@ class Protocol:
         self.cipher_out_name = None
         self.cipher_out_block_size = 0
         self.cipher_out_padding = INITIAL_PADDING
+        self._threading_lock = RLock()
         self._write_lock = Lock()
         self._write_thread = None
         self._read_thread = make_thread(self._read_thread_loop, "read", daemon=True)
@@ -211,16 +213,18 @@ class Protocol:
     def is_closed(self) -> bool:
         return self._closed
 
+    def is_sending_encrypted(self):
+        return self.cipher_out or self._conn.socktype in ("ssl", "wss", "ssh")
 
     def wait_for_io_threads_exit(self, timeout=None):
-        io_threads = [x for x in (self._read_thread, self._write_thread) if x is not None]
+        io_threads = (self._read_thread, self._write_thread)
         for t in io_threads:
-            if t.is_alive():
+            if t and t.is_alive():
                 t.join(timeout)
         exited = True
         cinfo = self._conn or "cleared connection"
         for t in io_threads:
-            if t.is_alive():
+            if t and t.is_alive():
                 log.warn("Warning: %s thread of %s is still alive (timeout=%s)", t.name, cinfo, timeout)
                 exited = False
         return exited
@@ -229,17 +233,21 @@ class Protocol:
         self._get_packet_cb = get_packet_cb
 
 
-    def set_cipher_in(self, ciphername, iv, password, key_salt, iterations, padding):
-        cryptolog("set_cipher_in%s", (ciphername, iv, password, key_salt, iterations))
-        self.cipher_in, self.cipher_in_block_size = get_decryptor(ciphername, iv, password, key_salt, iterations)
+    def set_cipher_in(self, ciphername, iv, password, key_salt, key_hash, key_size, iterations, padding):
+        cryptolog("set_cipher_in%s", (ciphername, iv, password, key_salt, key_hash, key_size, iterations))
+        self.cipher_in, self.cipher_in_block_size = get_decryptor(ciphername,
+                                                                  iv, password,
+                                                                  key_salt,key_hash, key_size, iterations)
         self.cipher_in_padding = padding
         if self.cipher_in_name!=ciphername:
             cryptolog.info("receiving data using %s encryption", ciphername)
             self.cipher_in_name = ciphername
 
-    def set_cipher_out(self, ciphername, iv, password, key_salt, iterations, padding):
-        cryptolog("set_cipher_out%s", (ciphername, iv, password, key_salt, iterations, padding))
-        self.cipher_out, self.cipher_out_block_size = get_encryptor(ciphername, iv, password, key_salt, iterations)
+    def set_cipher_out(self, ciphername, iv, password, key_salt, key_hash, key_size, iterations, padding):
+        cryptolog("set_cipher_out%s", (ciphername, iv, password, key_salt, key_hash, key_size, iterations, padding))
+        self.cipher_out, self.cipher_out_block_size = get_encryptor(ciphername,
+                                                                    iv, password,
+                                                                    key_salt, key_hash, key_size, iterations)
         self.cipher_out_padding = padding
         if self.cipher_out_name!=ciphername:
             cryptolog.info("sending data using %s encryption", ciphername)
@@ -257,9 +265,6 @@ class Protocol:
             self._write_format_thread,
             ) if x is not None)
 
-    def accept(self):
-        pass
-
     def parse_remote_caps(self, caps : typedict):
         for k,v in caps.dictget("aliases", {}).items():
             self.send_aliases[bytestostr(k)] = v
@@ -268,7 +273,7 @@ class Protocol:
 
     def get_info(self, alias_info=True) -> dict:
         info = {
-            "large_packets"         : tuple(bytestostr(x) for x in self.large_packets),
+            "large_packets"         : self.large_packets,
             "compression_level"     : self.compression_level,
             "max_packet_size"       : self.max_packet_size,
             "aliases"               : USE_ALIASES,
@@ -353,13 +358,14 @@ class Protocol:
         shm = self._source_has_more
         if not shm or self._closed:
             return
+        #from now on, take the shortcut:
+        self.source_has_more = shm.set
         shm.set()
         #start the format thread:
         if not self._write_format_thread and not self._closed:
-            self._write_format_thread = make_thread(self.write_format_thread_loop, "format", daemon=True)
-            self._write_format_thread.start()
-        #from now on, take shortcut:
-        self.source_has_more = self._source_has_more.set
+            with self._threading_lock:
+                assert not self._write_format_thread, "write format thread already started"
+                self._write_format_thread = start_thread(self.write_format_thread_loop, "format", daemon=True)
 
     def write_format_thread_loop(self):
         log("write_format_thread_loop starting")
@@ -404,7 +410,10 @@ class Protocol:
             if self.cipher_out:
                 proto_flags |= FLAGS_CIPHER
                 #note: since we are padding: l!=len(data)
-                padding_size = self.cipher_out_block_size - (payload_size % self.cipher_out_block_size)
+                if self.cipher_out_block_size==0:
+                    padding_size = 0
+                else:
+                    padding_size = self.cipher_out_block_size - (payload_size % self.cipher_out_block_size)
                 if padding_size==0:
                     padded = data
                 else:
@@ -455,7 +464,9 @@ class Protocol:
 
 
     def start_write_thread(self):
-        self._write_thread = start_thread(self._write_thread_loop, "write", daemon=True)
+        with self._threading_lock:
+            assert not self._write_thread, "write thread already started"
+            self._write_thread = start_thread(self._write_thread_loop, "write", daemon=True)
 
     def raw_write(self, packet_type, items, start_cb=None, end_cb=None, fail_cb=None, synchronous=True, more=False):
         """ Warning: this bypasses the compression and packet encoder! """
@@ -500,7 +511,7 @@ class Protocol:
         opts = compression.get_enabled_compressors(order=compression.PERFORMANCE_ORDER)
         compressors = caps.strtupleget("compressors")
         log("enable_compressor_from_caps(..) options=%s", opts)
-        for c in opts:      #ie: [zlib, lz4, lzo]
+        for c in opts:      #ie: [zlib, lz4]
             if c=="none":
                 continue
             if c in compressors or caps.boolget(c):
@@ -545,17 +556,17 @@ class Protocol:
                 l = len(item)
             except TypeError as e:
                 raise TypeError("invalid type %s in %s packet at index %s: %s" % (ti, packet[0], i, e)) from None
-            if ti==LargeStructure:
-                packet[i] = item.data
-                continue
-            if ti==Compressible:
+            if issubclass(ti, Compressible):
                 #this is a marker used to tell us we should compress it now
                 #(used by the client for clipboard data)
                 item = item.compress()
                 packet[i] = item
                 ti = type(item)
                 #(it may now be a "Compressed" item and be processed further)
-            if ti in (Compressed, LevelCompressed):
+            if issubclass(ti, LargeStructure):
+                packet[i] = item.data
+                continue
+            if issubclass(ti, Compressed):
                 #already compressed data (usually pixels, cursors, etc)
                 if not item.can_inline or l>INLINE_SIZE:
                     il = 0
@@ -602,7 +613,7 @@ class Protocol:
             packet[0] = packet_type
             verify_packet(packet)
             raise
-        if len(main_packet)>size_check and strtobytes(packet_in[0]) not in self.large_packets:
+        if len(main_packet)>size_check and bytestostr(packet_in[0]) not in self.large_packets:
             log.warn("Warning: found large packet")
             log.warn(" '%s' packet is %s bytes: ", packet_type, len(main_packet))
             log.warn(" argument types: %s", csv(type(x) for x in packet[1:]))
@@ -634,11 +645,11 @@ class Protocol:
             while not self._closed and callback():
                 pass
             log("io_thread_loop(%s, %s) loop ended, closed=%s", name, callback, self._closed)
-        except ConnectionClosedException:
-            log("%s closed", self._conn, exc_info=True)
+        except ConnectionClosedException as e:
+            log("%s closed in %s loop", self._conn, name, exc_info=True)
             if not self._closed:
                 #ConnectionClosedException means the warning has been logged already
-                self._connection_lost("%s connection %s closed" % (name, self._conn))
+                self._connection_lost(str(e))
         except (OSError, socket_error) as e:
             if not self._closed:
                 self._internal_error("%s connection %s reset" % (name, self._conn), e, exc_info=e.args[0] not in ABORT)
@@ -744,17 +755,17 @@ class Protocol:
 
     def _connection_lost(self, message="", exc_info=False):
         log("connection lost: %s", message, exc_info=exc_info)
-        self.close()
+        self.close(message)
         return False
 
 
     def invalid(self, msg, data):
-        self.idle_add(self._process_packet_cb, self, [Protocol.INVALID, msg, data])
+        self.idle_add(self._process_packet_cb, self, [INVALID, msg, data])
         # Then hang up:
         self.timeout_add(1000, self._connection_lost, msg)
 
     def gibberish(self, msg, data):
-        self.idle_add(self._process_packet_cb, self, [Protocol.GIBBERISH, msg, data])
+        self.idle_add(self._process_packet_cb, self, [GIBBERISH, msg, data])
         # Then hang up:
         self.timeout_add(self.hangup_delay, self._connection_lost, msg)
 
@@ -794,7 +805,9 @@ class Protocol:
         self._read_queue_put = self._read_queue.put
 
     def start_read_parser_thread(self):
-        self._read_parser_thread = start_thread(self._read_parse_thread_loop, "parse", daemon=True)
+        with self._threading_lock:
+            assert not self._read_parser_thread, "read parser thread already started"
+            self._read_parser_thread = start_thread(self._read_parse_thread_loop, "parse", daemon=True)
 
     def _read_parse_thread_loop(self):
         log("read_parse_thread_loop starting")
@@ -824,6 +837,7 @@ class Protocol:
         packet_index = 0
         compression_level = 0
         raw_packets = {}
+        PACKET_HEADER_CHAR = ord("P")
         while not self._closed:
             buf = self._read_queue.get()
             if not buf:
@@ -837,7 +851,7 @@ class Protocol:
                 if payload_size<0:
                     #try to handle the first buffer:
                     buf = read_buffers[0]
-                    if not header and buf[0]!=ord("P"):
+                    if not header and buf[0]!=PACKET_HEADER_CHAR:
                         self.invalid_header(self, buf, "invalid packet header byte")
                         return
                     #how much to we need to slice off to complete the header:
@@ -847,14 +861,13 @@ class Protocol:
                         #need to process more buffers to get a full header:
                         read_buffers.pop(0)
                         continue
-                    elif len(buf)>read:
-                        #got the full header and more, keep the rest of the packet:
-                        read_buffers[0] = buf[read:]
-                    else:
+                    if len(buf)<=read:
                         #we only got the header:
                         assert len(buf)==read
                         read_buffers.pop(0)
                         continue
+                    #got the full header and more, keep the rest of the packet:
+                    read_buffers[0] = buf[read:]
                     #parse the header:
                     # format: struct.pack(b'cBBBL', ...) - HEADER_SIZE bytes
                     _, protocol_flags, compression_level, packet_index, data_size = unpack_header(header)
@@ -865,13 +878,16 @@ class Protocol:
                         return
 
                     if protocol_flags & FLAGS_CIPHER:
-                        if self.cipher_in_block_size==0 or not self.cipher_in_name:
+                        if not self.cipher_in_name:
                             cryptolog.warn("Warning: received cipher block,")
                             cryptolog.warn(" but we don't have a cipher to decrypt it with,")
                             cryptolog.warn(" not an xpra client?")
                             self.invalid_header(self, header, "invalid encryption packet flag (no cipher configured)")
                             return
-                        padding_size = self.cipher_in_block_size - (data_size % self.cipher_in_block_size)
+                        if self.cipher_in_block_size==0:
+                            padding_size = 0
+                        else:
+                            padding_size = self.cipher_in_block_size - (data_size % self.cipher_in_block_size)
                         payload_size = data_size + padding_size
                     else:
                         #no cipher, no padding:
@@ -979,6 +995,9 @@ class Protocol:
                 #make sure we get a new header next time
                 header = b""
                 if packet_index>0:
+                    if packet_index in raw_packets:
+                        self.invalid("duplicate raw packet at index %i", packet_index)
+                        return
                     #raw packet, store it and continue:
                     raw_packets[packet_index] = data
                     payload_size = -1
@@ -992,16 +1011,17 @@ class Protocol:
                 except InvalidPacketEncodingException as e:
                     self.invalid("invalid packet encoding: %s" % e, data)
                     return
-                except ValueError as e:
+                except (ValueError, TypeError, IndexError) as e:
                     etype = packet_encoding.get_packet_encoding_type(protocol_flags)
                     log.error("Error parsing %s packet:", etype)
                     log.error(" %s", e)
                     if self._closed:
                         return
-                    log("failed to parse %s packet: %s", etype, hexstr(data[:128]))
-                    log(" %s", e)
-                    log(" data: %s", repr_ellipsized(data))
+                    log("failed to parse %s packet: %s", etype, hexstr(data[:128]), exc_info=True)
+                    data_str = memoryview_to_bytes(data)
+                    log(" data: %s", repr_ellipsized(data_str))
                     log(" packet index=%i, packet size=%i, buffer size=%s", packet_index, payload_size, bl)
+                    log(" full data: %s", hexstr(data_str))
                     self.gibberish("failed to parse %s packet" % etype, data)
                     return
 
@@ -1125,17 +1145,20 @@ class Protocol:
         log("flush_then_close: wait_for_write_lock()")
         wait_for_write_lock()
 
-    def close(self):
-        log("Protocol.close() closed=%s, connection=%s", self._closed, self._conn)
+    def close(self, message=None):
+        c = self._conn
+        log("Protocol.close(%s) closed=%s, connection=%s", message, self._closed, c)
         if self._closed:
             return
         self._closed = True
-        self.idle_add(self._process_packet_cb, self, [Protocol.CONNECTION_LOST])
-        c = self._conn
+        packet = [CONNECTION_LOST]
+        if message:
+            packet.append(message)
+        self.idle_add(self._process_packet_cb, self, packet)
         if c:
             self._conn = None
             try:
-                log("Protocol.close() calling %s", c.close)
+                log("Protocol.close(%s) calling %s", message, c.close)
                 c.close()
                 if self._log_stats is None and c.input_bytecount==0 and c.output_bytecount==0:
                     #no data sent or received, skip logging of stats:
@@ -1150,7 +1173,7 @@ class Protocol:
                 log.error("error closing %s", c, exc_info=True)
         self.terminate_queue_threads()
         self.idle_add(self.clean)
-        log("Protocol.close() done")
+        log("Protocol.close(%s) done", message)
 
     def steal_connection(self, read_callback=None):
         #so we can re-use this connection somewhere else
