@@ -139,6 +139,14 @@ def memoryview_to_bytes(v) -> bytes:
     return strtobytes(v)
 
 
+def set_proc_title(title):
+    try:
+        import setproctitle  #pylint: disable=import-outside-toplevel
+        setproctitle.setproctitle(title)  #@UndefinedVariable pylint: disable=c-extension-no-member
+    except ImportError as e:
+        get_util_logger().debug("setproctitle is not installed: %s", e)
+
+
 def getuid() -> int:
     if POSIX:
         return os.getuid()
@@ -266,7 +274,7 @@ def get_user_uuid() -> str:
     if user_uuid:
         return user_uuid
     import hashlib
-    u = hashlib.sha1()
+    u = hashlib.sha256()
     def uupdate(ustr):
         u.update(ustr.encode("utf-8"))
     uupdate(get_machine_id())
@@ -279,14 +287,6 @@ def get_user_uuid() -> str:
     return u.hexdigest()
 
 
-try:
-    from xpra.monotonic_time import _monotonic_time     #@UnresolvedImport
-    assert _monotonic_time()>0
-    monotonic_time = _monotonic_time
-except (ImportError, AssertionError):
-    monotonic_time = time.time
-
-
 def is_X11() -> bool:
     if OSX or WIN32:
         return False
@@ -297,9 +297,32 @@ def is_X11() -> bool:
         get_util_logger().debug("failed to load x11 bindings", exc_info=True)
         return True
 
-saved_env = os.environ.copy()
+def restore_script_env(env):
+    # On OSX PythonExecWrapper sets various env vars to point into the bundle
+    # and records the original variable contents. Here we revert them back
+    # to their original state in case any of those changes cause problems
+    # when running a command.
+    if "_PYTHON_WRAPPER_VARS" in env:
+        for v in env["_PYTHON_WRAPPER_VARS"].split():
+            origv = "_" + v
+            if origv in env:
+                env[v] = env[origv]
+            elif v in env:
+                del[v]
+            del env[origv]
+        del env["_PYTHON_WRAPPER_VARS"]
+    return env
+
+
+_saved_env = os.environ.copy()
+def get_saved_env():
+    return _saved_env.copy()
+
+def get_saved_env_var(var, default=None):
+    return _saved_env.get(var, default)
+
 def is_Wayland() -> bool:
-    return _is_Wayland(saved_env)
+    return _is_Wayland(_saved_env)
 
 def _is_Wayland(env : dict) -> bool:
     backend = env.get("GDK_BACKEND", "")
@@ -319,15 +342,16 @@ def is_distribution_variant(variant=b"Debian") -> bool:
     except Exception:
         pass
     try:
-        if variant==b"RedHat" and get_linux_distribution()[0].startswith(variant):
+        d = get_linux_distribution()[0]
+        if d==variant:
             return True
-        if get_linux_distribution()[0]==variant:
+        if variant==b"RedHat" and d.startswith(variant):
             return True
     except Exception:
         pass
     return False
 
-def get_distribution_version_id() -> bool:
+def get_distribution_version_id() -> str:
     if not POSIX:
         return ""
     try:
@@ -470,6 +494,7 @@ def load_binary_file(filename) -> bytes:
         with open(filename, "rb") as f:
             return f.read()
     except Exception as e:  # pragma: no cover
+        get_util_logger().debug("load_binary_file(%s)", filename, exc_info=True)
         get_util_logger().warn("Warning: failed to load '%s':", filename)
         get_util_logger().warn(" %s", e)
         return None
@@ -651,7 +676,7 @@ def path_permission_info(filename, ftype=None):
 #"** Message: pygobject_register_sinkfunc is deprecated (GstObject)"
 #ideally we would redirect to a buffer so we could still capture and show these messages in debug out
 class HideStdErr:
-
+    __slots__ = ("savedstderr")
     def __init__(self, *_args):
         self.savedstderr = None
 
@@ -671,7 +696,7 @@ class HideStdErr:
             os.dup2(self.savedstderr, 2)
 
 class HideSysArgv:
-
+    __slots__ = ("savedsysargv")
     def __init__(self, *_args):
         self.savedsysargv = None
 
@@ -685,11 +710,11 @@ class HideSysArgv:
 
 
 class OSEnvContext:
-
+    __slots__ = ("env")
     def __init__(self):
-        self.env = os.environ.copy()
+        self.env = {}
     def __enter__(self):
-        pass
+        self.env = os.environ.copy()
     def __exit__(self, *_args):
         os.environ.clear()
         os.environ.update(self.env)
@@ -698,7 +723,7 @@ class OSEnvContext:
 
 
 class FDChangeCaptureContext:
-
+    __slots__ = ("enter_fds", "exit_fds")
     def __init__(self):
         self.enter_fds = []
         self.exit_fds = []
@@ -714,18 +739,18 @@ class FDChangeCaptureContext:
         return sorted(tuple(set(self.enter_fds)-set(self.exit_fds)))
 
 class DummyContextManager:
-
+    __slots__ = ()
     def __enter__(self):
-        pass
+        """ do nothing """
     def __exit__(self, *_args):
-        pass
+        """ do nothing """
     def __repr__(self):
         return "DummyContextManager"
 
 
 #workaround incompatibility between paramiko and gssapi:
 class nomodule_context:
-
+    __slots__ = ("module_name", "saved_module")
     def __init__(self, module_name):
         self.module_name = module_name
     def __enter__(self):
@@ -741,7 +766,7 @@ class nomodule_context:
         return "nomodule_context(%s)" % self.module_name
 
 class umask_context:
-
+    __slots__ = ("umask", "orig_umask")
     def __init__(self, umask):
         self.umask = umask
     def __enter__(self):
@@ -786,7 +811,7 @@ def find_lib_ldconfig(libname):
             ldconfig = t
             break
     import subprocess
-    p = subprocess.Popen([ldconfig, "-p"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    p = subprocess.Popen("%s -p" % ldconfig, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
     data = bytestostr(p.communicate()[0])
 
     libpath = re.search(pattern, data, re.MULTILINE)        #@UndefinedVariable
@@ -800,8 +825,10 @@ def find_lib(libname):
     #it found the library in
     assert POSIX
     libpaths = os.environ.get("LD_LIBRARY_PATH", "").split(":")
-    libpaths.append("/usr/lib64")
-    libpaths.append("/usr/lib")
+    if BITS==64 and os.path.exists("/usr/lib64"):
+        libpaths.append("/usr/lib64")
+    else:
+        libpaths.append("/usr/lib")
     for libpath in libpaths:
         if not libpath or not os.path.exists(libpath):
             continue
@@ -812,9 +839,9 @@ def find_lib(libname):
 
 
 def pollwait(process, timeout=5):
-    start = monotonic_time()
+    start = time.monotonic()
     v = None
-    while monotonic_time()-start<timeout:
+    while time.monotonic()-start<timeout:
         v = process.poll()
         if v is not None:
             break
@@ -842,16 +869,19 @@ def which(command):
             return None
 
 def get_status_output(*args, **kwargs):
-    import subprocess
-    kwargs["stdout"] = subprocess.PIPE
-    kwargs["stderr"] = subprocess.PIPE
+    from subprocess import PIPE, Popen
+    kwargs.update({
+        "stdout"    : PIPE,
+        "stderr"    : PIPE,
+        "universal_newlines"    : True,
+        })
     try:
-        p = subprocess.Popen(*args, **kwargs)
+        p = Popen(*args, **kwargs)
     except Exception as e:
         print("error running %s,%s: %s" % (args, kwargs, e))
         return -1, "", ""
     stdout, stderr = p.communicate()
-    return p.returncode, stdout.decode("utf-8"), stderr.decode("utf-8")
+    return p.returncode, stdout, stderr
 
 
 def is_systemd_pid1() -> bool:
