@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,10 +9,11 @@ from xpra.util import envbool, csv
 from xpra.gtk_common.gtk_util import scaled_image
 from xpra.gtk_common.about import about
 from xpra.client.gtk_base.gtk_tray_menu_base import (
-    GTKTrayMenuBase, populate_encodingsmenu,
+    GTKTrayMenuBase,
     CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS,
     CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL,
     SHOW_UPLOAD, SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN,
+    SHOW_QR,
     )
 from xpra.platform.paths import get_icon
 from xpra.platform.darwin.gui import get_OSXApplication
@@ -185,6 +186,8 @@ class OSXMenuHelper(GTKTrayMenuBase):
         if SHOW_INFO_MENU:
             info_menu = self.make_menu()
             info_menu.append(self.make_sessioninfomenuitem())
+            if SHOW_QR:
+                info_menu.append(self.make_qrmenuitem())
             if SHOW_VERSION_CHECK:
                 info_menu.append(self.make_updatecheckmenuitem())
             info_menu.append(self.make_bugreportmenuitem())
@@ -218,11 +221,12 @@ class OSXMenuHelper(GTKTrayMenuBase):
         if mixin_features.windows and SHOW_ENCODINGS_MENU:
             encodings_menu = self.make_menu()
             def set_encodings_menu(*_args):
-                from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
+                from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER
                 server_encodings = list(self.client.server_encodings)
-                encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
+                encodings = [x for x in PREFERRED_ENCODING_ORDER if x in self.client.get_encodings()]
                 encodings.insert(0, "auto")
                 server_encodings.insert(0, "auto")
+                from xpra.client.gtk_base.menu_helper import populate_encodingsmenu
                 populate_encodingsmenu(encodings_menu, self.get_current_encoding, self.set_current_encoding, encodings, server_encodings)
             self.client.after_handshake(set_encodings_menu)
             menus.append(("Encoding", encodings_menu))
@@ -340,9 +344,9 @@ class OSXMenuHelper(GTKTrayMenuBase):
     #these methods are called by the superclass
     #but we don't have a quality or speed menu, so override and ignore
     def set_qualitymenu(self, *args):
-        pass
+        pass    #no quality menu on MacOS
     def set_speedmenu(self, *args):
-        pass
+        pass    #no speed menu on MacOS
 
     def _get_keyboard(self):
         if not self.client or not self.client.keyboard_helper:
