@@ -129,11 +129,14 @@ def do_get_default_log_dirs():
             dirs.append(os.path.join(x, "Logs", "Xpra"))
     except Exception:
         dirs.append("~/Library/Logs/Xpra")
-    dirs.append("/tmp")
+    import tempfile
+    dirs.append(tempfile.gettempdir())
     return dirs
 
 def do_get_socket_dirs():
-    return ["/var/tmp/%s-Xpra" % os.getuid(), "~/.xpra"]
+    #return ["/var/tmp/%s-Xpra" % os.getuid(), "~/.xpra"]
+    import tempfile
+    return [tempfile.gettempdir(), "~/.xpra"]
 
 def do_get_client_socket_dirs():
     return ["~/.xpra/clients"]
