@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -265,8 +265,7 @@ RegisterClassExW = user32.RegisterClassExW
 RegisterClassExW.restype = ATOM
 UnregisterClassA = user32.UnregisterClassA
 UnregisterClassA.restype = BOOL
-#painful to convert to LPCSTR, so don't bother:
-#UnregisterClassA.argtypes = [LPCSTR, HINSTANCE]
+UnregisterClassA.argtypes = [LPCSTR, HINSTANCE]
 UnregisterClassW = user32.UnregisterClassW
 UnregisterClassW.restype = BOOL
 UnregisterClassW.argtypes = [LPCWSTR, HINSTANCE]
