@@ -37,8 +37,9 @@ from xpra.platform.win32.common import (
     GetMonitorInfo,
     user32,
     )
+from xpra.common import KeyEvent
 from xpra.util import AdHocStruct, csv, envint, envbool, typedict
-from xpra.os_util import get_util_logger, strtobytes
+from xpra.os_util import get_util_logger
 from xpra.log import Logger
 
 log = Logger("win32")
@@ -243,10 +244,6 @@ def get_monitor_workarea_for_window(handle):
         return None
 
 
-def noop(*_args):
-    pass
-
-
 def get_window_handle(window):
     """ returns the win32 hwnd from a gtk.Window or gdk.Window """
     gdk_window = window
@@ -450,7 +447,7 @@ def fixup_window_style(self, *_args):
                 self.send_control_refresh(False)
             else:
                 self.send_control_refresh(True)
-    except:
+    except Exception:
         log.warn("failed to fixup window style", exc_info=True)
 
 def set_decorated(self, decorated):
@@ -509,7 +506,6 @@ def apply_maxsize_hints(window, hints):
     #which would remove the maximize button:
     for x in ("min_width", "min_height", "max_width", "max_height"):
         hints.pop(x, None)
-        hints.pop(strtobytes(x), None)
     window_state_updated(window)
 
 def apply_geometry_hints(self, hints):
@@ -528,18 +524,25 @@ def cache_pointer_offset(self, event):
     return gtk_x, gtk_y
 
 
+def no_set_group(*_args):
+    """ provide a dummy implementation """
+
 def add_window_hooks(window):
-    log("add_window_hooks(%s) WINDOW_HOOKS=%s, GROUP_LEADER=%s, UNDECORATED_STYLE=%s, MAX_SIZE_HINT=%s, MAX_SIZE_HINT=%s",
-            window, WINDOW_HOOKS, GROUP_LEADER, UNDECORATED_STYLE, MAX_SIZE_HINT, MAX_SIZE_HINT)
+    log("add_window_hooks(%s) WINDOW_HOOKS=%s, GROUP_LEADER=%s, UNDECORATED_STYLE=%s",
+            window, WINDOW_HOOKS, GROUP_LEADER, UNDECORATED_STYLE)
+    log(" MAX_SIZE_HINT=%s, MAX_SIZE_HINT=%s", MAX_SIZE_HINT, MAX_SIZE_HINT)
     if not WINDOW_HOOKS:
         #allows us to disable the win32 hooks for testing
         return
     try:
         gdk_window = window.get_window()
-        #win32 cannot use set_group by default:
-        gdk_window.set_group = noop
-    except:
+    except Exception:
         gdk_window = None
+    if not gdk_window:
+        #can't get a handle from a None value...
+        return
+    #at least provide a dummy method:
+    gdk_window.set_group = no_set_group
     handle = get_window_handle(gdk_window)
     if not handle:
         log.warn("Warning: cannot add window hooks without a window handle!")
@@ -608,7 +611,8 @@ def add_window_hooks(window):
                 units = distance / WHEEL_DELTA
                 client = getattr(window, "_client")
                 wid = getattr(window, "_id", 0)
-                mouselog("win32 mousewheel: orientation=%s, distance=%i, wheel-delta=%s, units=%.3f, new value=%.1f, keys=%#x, x=%i, y=%i, client=%s, wid=%i", orientation, distance, WHEEL_DELTA, units, distance, keys, x, y, client, wid)
+                mouselog("win32 mousewheel: orientation=%s, distance=%i, wheel-delta=%s, units=%.3f, new value=%.1f, keys=%#x, x=%i, y=%i, client=%s, wid=%i",
+                         orientation, distance, WHEEL_DELTA, units, distance, keys, x, y, client, wid)
                 if client and wid>0:
                     if orientation==VERTICAL:
                         deltax = 0
@@ -616,16 +620,15 @@ def add_window_hooks(window):
                     else:
                         deltax = units
                         deltay = 0
-                    client.wheel_event(wid, deltax, deltay)
+                    pointer = window.get_mouse_position()
+                    client.wheel_event(wid, deltax, deltay, pointer)
             def mousewheel(_hwnd, _event, wParam, lParam):
                 handle_wheel(VERTICAL, wParam, lParam)
                 return 0
             def mousehwheel(_hwnd, _event, wParam, lParam):
                 handle_wheel(HORIZONTAL, wParam, lParam)
                 return 0
-            WM_MOUSEHWHEEL = 0x020E
             win32hooks.add_window_event_handler(win32con.WM_MOUSEWHEEL, mousewheel)
-            win32hooks.add_window_event_handler(WM_MOUSEHWHEEL, mousehwheel)
 
 
 def remove_window_hooks(window):
@@ -991,7 +994,7 @@ class ClientExtras:
             start_thread(self.init_keyboard_listener, "keyboard-listener", daemon=True)
 
     def ready(self):
-        pass
+        """ nothing specific to do here """
 
     def cleanup(self):
         log("ClientExtras.cleanup()")
@@ -1017,8 +1020,6 @@ class ClientExtras:
         #self.client = None
 
     def init_keyboard_listener(self):
-        class WindowsKeyEvent(AdHocStruct):
-            pass
         class KBDLLHOOKSTRUCT(Structure):
             _fields_ = [("vk_code", DWORD),
                         ("scan_code", DWORD),
@@ -1060,7 +1061,7 @@ class ClientExtras:
                                 try:
                                     keycode = int(k)
                                     break
-                                except:
+                                except ValueError:
                                     pass
                             if keycode>0:
                                 break
@@ -1081,7 +1082,7 @@ class ClientExtras:
                     #keylog.info("keyboard helper=%s, modifier keycodes=%s", kh, modifier_keycodes)
                     grablog("vk_code=%s, scan_code=%s, event=%s, keyname=%s, keycode=%s, modifiers=%s, focused=%s", vk_code, scan_code, ALL_KEY_EVENTS.get(wParam), keyname, keycode, modifiers, focused)
                     if keycode>0:
-                        key_event = WindowsKeyEvent()
+                        key_event = KeyEvent()
                         key_event.keyname = keyname
                         key_event.pressed = wParam in DOWN
                         key_event.modifiers = modifiers
