@@ -7,6 +7,7 @@
 import os.path
 import sys
 import ctypes
+import tempfile
 import platform
 
 from xpra.os_util import get_util_logger
@@ -17,12 +18,16 @@ SHGetFolderPath = shell32.SHGetFolderPathW
 CSIDL_APPDATA = 26
 CSIDL_LOCAL_APPDATA = 28
 CSIDL_COMMON_APPDATA = 35
+CSIDL_PROFILE = 40
+
 
 def sh_get_folder_path(v):
     try:
         buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
         SHGetFolderPath(0, v, None, 0, buf)
-        return buf.value
+        if not buf.value:
+            return buf.value
+        return os.path.normpath(buf.value)
     except Exception:
         return None
 
@@ -38,7 +43,7 @@ def _get_data_dir(roaming=True):
         appdata = os.environ.get("APPDATA" if roaming else "LOCALAPPDATA")
     if not appdata:
         #we need some kind of path..
-        appdata = os.environ.get("TEMP", "C:\\TEMP\\")
+        appdata = tempfile.gettempdir()
         assert appdata, "cannot find any usable directory for log files"
     if not os.path.exists(appdata):
         os.mkdir(appdata)
@@ -64,7 +69,7 @@ def do_get_icon_dir():
 
 def do_get_default_log_dirs():
     dd = _get_data_dir()
-    temp = os.environ.get("TEMP", "C:\\TEMP\\")
+    temp = tempfile.gettempdir()
     if dd==temp:
         return [temp]
     return [dd, temp]
@@ -73,10 +78,7 @@ def do_get_default_log_dirs():
 def get_program_data_dir():
     #ie: "C:\ProgramData"
     try:
-        buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
-        SHGetFolderPath(0, CSIDL_COMMON_APPDATA, None, 0, buf)
-        if buf.value:
-            return buf.value
+        return sh_get_folder_path(CSIDL_COMMON_APPDATA) or "C:\\ProgramData"
     except Exception:
         get_util_logger().debug("get_program_data_dir()", exc_info=True)
     return "C:\\ProgramData"
@@ -85,6 +87,18 @@ def do_get_system_conf_dirs():
     #ie: C:\ProgramData\Xpra
     return [os.path.join(get_program_data_dir(), "Xpra")]
 
+
+def do_get_ssl_cert_dirs():
+    dirs = []
+    for i in (CSIDL_PROFILE, CSIDL_COMMON_APPDATA, CSIDL_LOCAL_APPDATA, CSIDL_APPDATA):
+        d = os.path.join(sh_get_folder_path(i), "Xpra")
+        dirs.append(d)
+        d = os.path.join(sh_get_folder_path(i), "Xpra", "ssl")
+        dirs.append(d)
+    dirs += do_get_default_conf_dirs()
+    return dirs
+
+
 def do_get_ssh_conf_dirs():
     if platform.architecture()[0]=="32bit":
         system32 = "SysNative"
@@ -92,19 +106,26 @@ def do_get_ssh_conf_dirs():
         system32 = "System32"
     windows_dir = os.environ.get("SystemRoot", os.environ.get("WINDIR", "C:\\Windows"))
     openssh_dir = os.path.join(windows_dir, system32, "OpenSSH")
-    return [
-        os.path.join(get_program_data_dir(), "ssh"),    #ie: C:\ProgramData\ssh
-        "%APPDATA%\\ssh",   #ie: C:\Users\Username\AppData\Roaming\ssh
+    dirs = []
+    for i in (CSIDL_PROFILE, CSIDL_COMMON_APPDATA, CSIDL_LOCAL_APPDATA, CSIDL_APPDATA):
+        d = os.path.join(sh_get_folder_path(i), "SSH")
+        dirs.append(d)
+    dirs += do_get_default_conf_dirs()+[
         openssh_dir,        #ie: C:\Windows\system32\OpenSSH
         "~/.ssh",
         "~/ssh",
         ]
+    return dirs
 
 def do_get_ssh_known_hosts_files():
     #reverse the order (avoid dotfiles on win32):
     return ("~/ssh/known_hosts", "~/.ssh/known_hosts")
 
 
+def do_get_sessions_dir():
+    return "%APPDATA%\\Xpra"
+
+
 def do_get_default_conf_dirs():
     #ie: C:\Program Files\Xpra\
     from xpra.platform.paths import get_app_dir
