@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.util import std, typedict
+from xpra.util import std, typedict, net_utf8
 from xpra.server.source.stub_source_mixin import StubSourceMixin
 from xpra.os_util import platform_name
 from xpra.log import Logger
@@ -12,10 +12,10 @@ from xpra.log import Logger
 log = Logger("server")
 
 
-"""
-Store information about the client.
-"""
 class ClientInfoMixin(StubSourceMixin):
+    """
+    Store information about the client.
+    """
 
     def cleanup(self):
         self.init_state()
@@ -26,6 +26,7 @@ class ClientInfoMixin(StubSourceMixin):
         self.machine_id = ""
         self.hostname = ""
         self.username = ""
+        self.user = ""
         self.name = ""
         self.argv = ()
         self.sharing = False
@@ -58,6 +59,7 @@ class ClientInfoMixin(StubSourceMixin):
         self.machine_id = c.strget("machine_id")
         self.hostname = c.strget("hostname")
         self.username = c.strget("username")
+        self.user = c.strget("user")
         self.name = c.strget("name")
         self.argv = c.strtupleget("argv")
         self.sharing = c.boolget("share")
@@ -71,7 +73,7 @@ class ClientInfoMixin(StubSourceMixin):
         self.client_revision = c.strget("build.revision")
         self.client_bits = c.intget("python.bits")
         self.client_proxy = c.boolget("proxy")
-        self.client_wm_name = c.strget("wm_name")
+        self.client_wm_name = c.conv_get("wm_name", "", net_utf8)
         self.client_session_type = c.strget("session-type")
         self.client_session_type_full = c.strget("session-type.full", "")
         self.client_setting_change = c.boolget("setting-change")
@@ -138,13 +140,15 @@ class ClientInfoMixin(StubSourceMixin):
                 "version"           : self.client_version or "unknown",
                 "revision"          : self.client_revision or "unknown",
                 "platform_name"     : platform_name(self.client_platform, self.client_release),
-                "session-type"      : self.client_session_type,
-                "session-type.full" : self.client_session_type_full,
-                "session-id"        : self.session_id,
-                "uuid"              : self.uuid,
+                "session-type"      : self.client_session_type or "",
+                "session-type.full" : self.client_session_type_full or "",
+                "session-id"        : self.session_id or "",
+                "uuid"              : self.uuid or "",
                 "hostname"          : self.hostname or "",
-                "argv"              : self.argv,
-                "sharing"           : self.sharing,
+                "user"              : self.user,
+                "name"              : self.name,
+                "argv"              : self.argv or (),
+                "sharing"           : bool(self.sharing),
                 }
 
         def addattr(k, name):
