@@ -7,10 +7,10 @@
 # later version. See the file COPYING for details.
 
 from math import sqrt
+from time import monotonic
 
 from collections import deque
 from xpra.simple_stats import get_list_stats, get_weighted_list_stats
-from xpra.os_util import monotonic_time
 from xpra.util import engs, csv, envint
 from xpra.server.cystats import (logp,      #@UnresolvedImport
     calculate_time_weighted_average,        #@UnresolvedImport
@@ -24,7 +24,7 @@ log = Logger("stats")
 
 #how many historical records to keep
 #for the various statistics we collect:
-#(cannot be lower than DamageBatchConfig.MAX_EVENTS)
+#(cannot be lower than DamageBatchConfig.max_events)
 NRECS = 100
 
 TARGET_LATENCY_TOLERANCE = envint("XPRA_TARGET_LATENCY_TOLERANCE", 20)/1000.0
@@ -43,7 +43,7 @@ class WindowPerformanceStatistics:
     DEFAULT_TARGET_LATENCY = 0.1
 
     def reset(self):
-        self.init_time = monotonic_time()
+        self.init_time = monotonic()
         self.client_decode_time = deque(maxlen=NRECS)       #records how long it took the client to decode frames:
                                                             #(ack_time, no of pixels, decoding_time*1000*1000)
         self.encoding_stats = deque(maxlen=NRECS)           #encoding: (time, coding, pixels, bpp, compressed_size, encoding_time)
@@ -64,7 +64,9 @@ class WindowPerformanceStatistics:
         self.damage_events_count = 0
         self.packet_count = 0
 
+        self.resize_events = deque(maxlen=4)                #(time)
         self.last_resized = 0
+        self.last_packet_time = 0
 
         #these values are calculated from the values above (see update_averages)
         self.target_latency = self.DEFAULT_TARGET_LATENCY
@@ -137,7 +139,7 @@ class WindowPerformanceStatistics:
             #If nothing happens for a while then we can reduce the batch delay,
             #however we must ensure this is not caused by a high system latency
             #so we ignore short elapsed times.
-            elapsed = monotonic_time()-ldet
+            elapsed = monotonic()-ldet
             mtime = max(0, elapsed-self.max_latency*2)
             #the longer the time, the more we slash:
             weight = sqrt(mtime)
@@ -149,7 +151,7 @@ class WindowPerformanceStatistics:
         if bandwidth_limit>0:
             #calculate how much bandwith we have used in the last second (in bps):
             #encoding_stats.append((end, coding, w*h, bpp, len(data), end-start))
-            cutoff = monotonic_time()-1
+            cutoff = monotonic()-1
             used = sum(v[4] for v in tuple(self.encoding_stats) if v[0]>cutoff) * 8
             info = {
                 "budget"  : bandwidth_limit,
@@ -173,7 +175,6 @@ class WindowPerformanceStatistics:
         #encoding stats:
         estats = tuple(self.encoding_stats)
         if estats:
-            encodings_used = [x[1] for x in estats]
             def add_compression_stats(enc_stats, encoding=None):
                 comp_ratios_pct = []
                 comp_times_ns = []
@@ -194,8 +195,9 @@ class WindowPerformanceStatistics:
                 if total_time>0:
                     einfo["pixels_encoded_per_second"] = int(total_pixels / total_time)
             add_compression_stats(estats)
+            encodings_used = tuple(x[1] for x in estats)
             for encoding in encodings_used:
-                enc_stats = [x for x in estats if x[1]==encoding]
+                enc_stats = tuple(x for x in estats if x[1]==encoding)
                 add_compression_stats(enc_stats, encoding)
 
         dinfo = info.setdefault("damage", {})
@@ -213,7 +215,7 @@ class WindowPerformanceStatistics:
         return info
 
 
-    def get_target_client_latency(self, min_client_latency, avg_client_latency, abs_min=0.010):
+    def get_target_client_latency(self, min_client_latency, avg_client_latency, abs_min=0.010, jitter=0):
         """ geometric mean of the minimum (+20%) and average latency
             but not higher than twice more than the minimum,
             and not lower than abs_min.
@@ -225,16 +227,16 @@ class WindowPerformanceStatistics:
             decoding_latency = calculate_timesize_weighted_average(cdt)[0]/1000.0
         min_latency = max(abs_min, min_client_latency or abs_min)*1.2
         avg_latency = max(min_latency, avg_client_latency or abs_min)
-        max_latency = 2.0*min_latency
-        return max(abs_min, min(max_latency, sqrt(min_latency*avg_latency))) + decoding_latency
+        max_latency = min(avg_latency, 4.0*min_latency+0.100)
+        return max(abs_min, min(max_latency, sqrt(min_latency*avg_latency))) + decoding_latency + jitter/1000.0
 
     def get_client_backlog(self):
         packets_backlog, pixels_backlog, bytes_backlog = 0, 0, 0
         if self.damage_ack_pending:
-            sent_before = monotonic_time()-(self.target_latency+TARGET_LATENCY_TOLERANCE)
-            dropped_acks_time = monotonic_time()-60      #1 minute
+            sent_before = monotonic()-(self.target_latency+TARGET_LATENCY_TOLERANCE)
+            dropped_acks_time = monotonic()-60      #1 minute
             drop_missing_acks = []
-            for sequence, item in self.damage_ack_pending.items():
+            for sequence, item in tuple(self.damage_ack_pending.items()):
                 start_send_at = item[0]
                 end_send_at = item[3]
                 if end_send_at==0 or start_send_at>sent_before:
@@ -255,25 +257,21 @@ class WindowPerformanceStatistics:
                 log.error(" connection may be closed or closing,")
                 log.error(" sequence numbers missing: %s", csv(drop_missing_acks))
                 for sequence in drop_missing_acks:
-                    try:
-                        del self.damage_ack_pending[sequence]
-                    except KeyError:
-                        pass
+                    self.damage_ack_pending.pop(sequence, None)
         return packets_backlog, pixels_backlog, bytes_backlog
 
     def get_acks_pending(self):
         return sum(1 for x in self.damage_ack_pending.values() if x[0]!=0)
 
-    def get_packets_backlog(self, latency_tolerance_pct=100):
-        packets_backlog = 0
-        if self.damage_ack_pending:
-            sent_before = monotonic_time()-((self.target_latency+0.020)*latency_tolerance_pct/100.0)
-            for item in self.damage_ack_pending.values():
-                start_send_at = item[0]
-                end_send_at = item[3]
-                if end_send_at>0 and start_send_at<=sent_before:
-                    packets_backlog += 1
-        return packets_backlog
+    def get_late_acks(self, latency):
+        now = monotonic()
+        sent_before = now-latency
+        #start_send_at = item[0]
+        #end_send_at = item[3]
+        late = sum(1 for item in self.damage_ack_pending.values() if item[3]>0 and item[0]<=sent_before)
+        log("get_late_acks(%i)=%i (%i in full pending list)",
+            1000*latency, late, len(self.damage_ack_pending))
+        return late
 
     def get_pixels_encoding_backlog(self):
         pixels, count = 0, 0
@@ -283,7 +281,7 @@ class WindowPerformanceStatistics:
         return pixels, count
 
     def get_bitrate(self, max_elapsed=1):
-        cutoff = monotonic_time()-max_elapsed
+        cutoff = monotonic()-max_elapsed
         recs = tuple((v[0], v[4]) for v in tuple(self.encoding_stats) if v[0]>=cutoff)
         if len(recs)<2:
             return 0
@@ -294,5 +292,5 @@ class WindowPerformanceStatistics:
         return int(bits/elapsed)
 
     def get_damage_pixels(self, elapsed=1):
-        cutoff = monotonic_time()-elapsed
+        cutoff = monotonic()-elapsed
         return sum(v[3]*v[4] for v in tuple(self.last_damage_events) if v[0]>cutoff)
