@@ -27,7 +27,7 @@ from xpra.x11.bindings.window_bindings import ( #@UnresolvedImport
     constants, PropertyError,                   #@UnresolvedImport
     X11WindowBindings,                          #@UnresolvedImport
     )
-from xpra.os_util import bytestostr
+from xpra.os_util import bytestostr, strtobytes
 from xpra.util import csv, repr_ellipsized, ellipsizer, first_time
 from xpra.log import Logger
 
@@ -72,7 +72,7 @@ log("TRANSLATED_TARGETS=%s", TRANSLATED_TARGETS)
 def xatoms_to_strings(data):
     l = len(data)
     if l%sizeof_long!=0:
-        raise Exception("invalid length for atom array: %i, value=%s" % (l, repr_ellipsized(str(data))))
+        raise ValueError("invalid length for atom array: %i, value=%s" % (l, repr_ellipsized(str(data))))
     natoms = l//sizeof_long
     atoms = struct.unpack(b"@"+b"L"*natoms, data)
     with xsync:
@@ -109,7 +109,9 @@ class X11Clipboard(ClipboardTimeoutHelper, GObject.GObject):
 
     def init_window(self):
         root = get_default_root_window()
-        self.window = GDKX11Window(root, width=1, height=1, title="Xpra-Clipboard", wclass=Gdk.WindowWindowClass.INPUT_ONLY)
+        self.window = GDKX11Window(root, width=1, height=1,
+                                   title="Xpra-Clipboard",
+                                   wclass=Gdk.WindowWindowClass.INPUT_ONLY)
         self.window.set_events(Gdk.EventMask.PROPERTY_CHANGE_MASK | self.window.get_events())
         xid = self.window.get_xid()
         with xsync:
@@ -199,12 +201,12 @@ class X11Clipboard(ClipboardTimeoutHelper, GObject.GObject):
 
     def _munge_raw_selection_to_wire(self, target, dtype, dformat, data):
         if dformat==32 and dtype in ("ATOM", "ATOM_PAIR"):
-            return "atoms", _filter_targets(xatoms_to_strings(data))
+            return "atoms", self.remote_targets(xatoms_to_strings(data))
         return super()._munge_raw_selection_to_wire(target, dtype, dformat, data)
 
     def _munge_wire_selection_to_raw(self, encoding, dtype, dformat, data):
         if encoding=="atoms":
-            return strings_to_xatoms(_filter_targets(data))
+            return strings_to_xatoms(self.local_targets(data))
         return super()._munge_wire_selection_to_raw(encoding, dtype, dformat, data)
 
 GObject.type_register(X11Clipboard)
@@ -258,9 +260,6 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
         for target in lr:
             self.got_local_contents(target)
 
-    def init_uuid(self):
-        ClipboardProxyCore.init_uuid(self)
-        self.claim()
 
     def got_token(self, targets, target_data=None, claim=True, synchronous_client=False):
         # the remote end now owns the clipboard
@@ -269,7 +268,7 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
             return
         self._got_token_events += 1
         log("got token, selection=%s, targets=%s, target data=%s, claim=%s, can-receive=%s",
-            self._selection, targets, target_data, claim, self._can_receive)
+            self._selection, targets, ellipsizer(target_data), claim, self._can_receive)
         if claim:
             self._have_token = True
         if self._can_receive:
@@ -463,6 +462,10 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
                 return
             with xsync:
                 if data is not None:
+                    if isinstance(data, str):
+                        #the data is already in the correct format,
+                        #but the cython bindings require real 'bytes'
+                        data = strtobytes(data)
                     X11Window.XChangeProperty(xid, prop, dtype, dformat, data)
                 else:
                     #maybe even delete the property?
@@ -513,7 +516,7 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
         self.do_owner_changed()
         self.schedule_emit_token()
 
-    def schedule_emit_token(self):
+    def schedule_emit_token(self, min_delay=0):
         if not (self._want_targets or self._greedy_client):
             self._have_token = False
             self.emit("send-clipboard-token", ())
