@@ -9,18 +9,19 @@
 import os
 import signal
 import math
+from time import monotonic
 from collections import deque
 from gi.repository import GObject, Gtk, Gdk, GdkX11
 
 from xpra.version_util import XPRA_VERSION
-from xpra.util import updict, rindex, envbool, envint, typedict, AdHocStruct, WORKSPACE_NAMES
-from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, monotonic_time
+from xpra.util import net_utf8, updict, rindex, envbool, envint, typedict, WORKSPACE_NAMES
+from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr
 from xpra.common import CLOBBER_UPGRADE, MAX_WINDOW_SIZE
-from xpra.server import server_features
+from xpra.server import server_features, EXITING_CODE
 from xpra.gtk_common.gobject_util import one_arg_signal
 from xpra.gtk_common.gtk_util import get_default_root_window, get_pixbuf_from_data
 from xpra.x11.common import Unmanageable
-from xpra.x11.gtk_x11.prop import prop_set
+from xpra.x11.gtk_x11.prop import raw_prop_set
 from xpra.x11.gtk_x11.tray import get_tray_window, SystemTray
 from xpra.x11.gtk_x11.selection import AlreadyOwned
 from xpra.x11.gtk_x11.gdk_bindings import (
@@ -59,6 +60,7 @@ WINDOW_SIGNALS = os.environ.get("XPRA_WINDOW_SIGNALS", "SIGINT,SIGTERM,SIGQUIT,S
 
 
 class DesktopState:
+    __slots__ = ("geom", "resize_counter", "shown")
     def __init__(self, geom, resize_counter=0, shown=False):
         self.geom = geom
         self.resize_counter = resize_counter
@@ -67,9 +69,10 @@ class DesktopState:
         return "DesktopState(%s, %i, %s)" % (self.geom, self.resize_counter, self.shown)
 
 class DesktopManager(Gtk.Widget):
+    __slots__ = ("_models")
     def __init__(self):
         self._models = {}
-        Gtk.Widget.__init__(self)
+        super().__init__()
         self.set_property("can-focus", True)
         self.realize()
 
@@ -241,20 +244,15 @@ class XpraServer(GObject.GObject, X11ServerBase):
         # selecting SubstructureRedirect.
         root = get_default_root_window()
         root.set_events(root.get_events() | Gdk.EventMask.SUBSTRUCTURE_MASK)
-        prop_set(root, "XPRA_SERVER", "latin1", strtobytes(XPRA_VERSION).decode())
+        xid = root.get_xid()
+        raw_prop_set(xid, "XPRA_SERVER", "latin1", strtobytes(XPRA_VERSION).decode())
         add_event_receiver(root, self)
         if self.sync_xvfb>0:
-            xid = root.get_xid()
             try:
                 with xsync:
                     self.root_overlay = X11Window.XCompositeGetOverlayWindow(xid)
                     if self.root_overlay:
-                        #ugly: API expects a window object with a ".get_xid()" method
-                        window = AdHocStruct()
-                        def get_xid():
-                            return self.root_overlay
-                        window.get_xid = root.get_xid
-                        prop_set(window, "WM_TITLE", "latin1", "RootOverlay")
+                        raw_prop_set(self.root_overlay, "WM_TITLE", "latin1", "RootOverlay")
                         X11Window.AllowInputPassthrough(self.root_overlay)
             except Exception as e:
                 log("XCompositeGetOverlayWindow(%#x)", xid, exc_info=True)
@@ -274,10 +272,12 @@ class XpraServer(GObject.GObject, X11ServerBase):
                 log("Error: cannot create our window manager", exc_info=True)
                 display = os.environ.get("DISPLAY", "")
                 #make sure we don't kill the vfb since we don't own it:
-                from xpra.server import EXITING_CODE
                 self._upgrading = EXITING_CODE
                 from xpra.scripts.config import InitException  #pylint: disable=import-outside-toplevel
-                raise InitException("another window manager is active on display '%s'" % display) from None
+                from xpra.platform.xposix.gui import get_x11_wm_name
+                wm_name = get_x11_wm_name() or "another window manager"
+                err = "%s is already active on display %r" % (wm_name, display)
+                raise InitException(err) from None
         if server_features.windows:
             self._wm.connect("new-window", self._new_window_signaled)
         self._wm.connect("quit", lambda _: self.clean_quit(True))
@@ -307,6 +307,10 @@ class XpraServer(GObject.GObject, X11ServerBase):
             self.X11_ungrab()
         X11ServerBase.do_cleanup(self)
 
+    def clean_x11_properties(self):
+        super().clean_x11_properties()
+        self.do_clean_x11_properties("XPRA_SERVER")
+
 
     def last_client_exited(self):
         #last client is gone:
@@ -388,7 +392,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
                      })
         try:
             info["client-geometry"] = self._desktop_manager.window_geometry(window)
-        except:
+        except KeyError:
             pass        #OR or tray window
         return info
 
@@ -404,10 +408,26 @@ class XpraServer(GObject.GObject, X11ServerBase):
         if not self._server_sources:
             super().set_screen_geometry_attributes(w, h)
 
-    def set_desktops(self, names):
+    def calculate_desktops(self):
         wm = self._wm
-        if wm:
-            wm.set_desktop_list(names)
+        if not wm:
+            return
+        count = 1
+        sources = tuple(self._server_sources.values())
+        for ss in sources:
+            if ss.desktops:
+                count = max(count, ss.desktops)
+        count = max(1, min(20, count))
+        names = []
+        for i in range(count):
+            name = "Main" if i==0 else "Desktop %i" % (i+1)
+            for ss in sources:
+                if ss.desktops and i<len(ss.desktop_names) and ss.desktop_names[i]:
+                    v = net_utf8(ss.desktop_names[i])
+                    if v!="0" or i!=0:
+                        name = v
+            names.append(name)
+        wm.set_desktop_list(names)
 
     def set_workarea(self, workarea):
         wm = self._wm
@@ -458,8 +478,14 @@ class XpraServer(GObject.GObject, X11ServerBase):
             self._add_new_window(window)
 
         root = get_default_root_window()
-        with xsync:
-            children = get_children(root)
+        try:
+            with xsync:
+                children = get_children(root)
+        except XError:
+            log("load_existing_windows()", exc_info=True)
+            log("trying again")
+            with xsync:
+                children = get_children(root)
         for window in children:
             xid = window.get_xid()
             can_add = False
@@ -568,7 +594,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         if window_prop_set:
             try:
                 window_prop_set("_XPRA_WID", "u32", wid)
-            except:
+            except XError:
                 pass    #this can fail if the window disappears
                 #but we don't really care, it will get cleaned up soon enough
         return wid
@@ -612,7 +638,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         #TODO: find a better way to choose the timer delay:
         #for now, we wait at least 100ms, up to 250ms if the client has just sent us a resize:
         #(lcce should always be in the past, so min(..) should be redundant here)
-        delay = max(100, min(250, 250 + 1000 * (lcce-monotonic_time())))
+        delay = max(100, min(250, 250 + 1000 * (lcce-monotonic())))
         self.snc_timer = self.timeout_add(int(delay), self.size_notify_clients, window, lcce)
 
     def size_notify_clients(self, window, lcce=-1):
@@ -860,12 +886,13 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
 
     def _set_window_state(self, proto, wid, window, new_window_state):
-        assert proto in self._server_sources
+        if proto not in self._server_sources:
+            return []
         if not new_window_state:
             return []
         nws = typedict(new_window_state)
         metadatalog("set_window_state%s", (wid, window, new_window_state))
-        changes = []
+        changes = {}
         if "frame" in new_window_state:
             #the size of the window frame may have changed
             frame = nws.inttupleget("frame", (0, 0, 0, 0))
@@ -878,7 +905,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
             else:
                 if window.get_property("iconic")!=bool(iconified):
                     window.set_property("iconic", iconified)
-                    changes.append("iconified")
+                    changes["iconified"] = bool(iconified)
         #handle wm_state virtual booleans:
         for k in (
             "maximized", "above",
@@ -894,9 +921,9 @@ class XpraServer(GObject.GObject, X11ServerBase):
             #metadatalog.info("set window state for '%s': current state=%s, new state=%s", k, cur_state, new_state)
             if cur_state!=new_state:
                 window.update_wm_state(k, new_state)
-                changes.append(k)
+                changes[k] = new_state
         metadatalog("set_window_state: changes=%s", changes)
-        return changes
+        return tuple(changes.keys())
 
 
     def get_window_position(self, window):
@@ -1027,7 +1054,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
                     damage = True
             else:
                 assert skip_geometry or not window.is_OR(), "received a configure packet with geometry for OR window %s from %s: %s" % (window, proto, packet)
-                self.last_client_configure_event = monotonic_time()
+                self.last_client_configure_event = monotonic()
                 if is_ui_driver and len(packet)>=9:
                     changes = self._set_window_state(proto, wid, window, packet[8])
                     if changes:
@@ -1093,10 +1120,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         if workspace is not None:
             window.move_to_workspace(workspace)
             #we have handled it on the window directly, so remove it from client properties
-            try:
-                del new_client_properties[b"workspace"]
-            except KeyError:
-                del new_client_properties["workspace"]
+            new_client_properties.pop("workspace", None)
         #handle the rest as normal:
         super()._set_client_properties(proto, wid, window, new_client_properties)
 
@@ -1117,7 +1141,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
 
     def _process_close_window(self, proto, packet):
-        assert proto in self._server_sources
+        if proto not in self._server_sources:
+            return
         wid = packet[1]
         window = self._lookup_window(wid)
         windowlog("client closed window %s - %s", wid, window)
@@ -1129,7 +1154,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
 
     def _process_window_signal(self, proto, packet):
-        assert proto in self._server_sources
+        if proto not in self._server_sources:
+            return
         wid = packet[1]
         sig = bytestostr(packet[2])
         if sig not in WINDOW_SIGNALS:
@@ -1283,7 +1309,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
             if prio==0:
                 try:
                     prio = rindex(focus_history, wid)
-                except:
+                except ValueError:
                     pass        #not in focus history!
             order[(prio, wid)] = window
         log("do_repaint_root_overlay() has_focus=%s, has_grab=%s, windows in order=%s",
